
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "globals.h"
#include "sym.h"

struct caserec {
	struct caserec *next;
	int label;
	int caseval;
	int uniq;
};

struct blockctx {
	struct blockctx *outer;
	struct caserec *cases;
	int uniq;
	int deflabel;
	unsigned is_switch:1,
		 has_default:1,
		 :0;
};

static struct blockctx *blockctx;

struct blockctx *
add_switch_case(caseval, is_default)
{
	struct blockctx *ctx;
	struct caserec *p;

	for (ctx = blockctx; ctx; ctx = ctx->outer)
		if (ctx->is_switch)
			break;
	if (!ctx)
		error("No switch active");

	if (is_default) {
		if (ctx->has_default)
			error("Duplicate default in switch");
		ctx->has_default = 1;
		ctx->deflabel = uniq++;
printf("L%d:\n", ctx->deflabel);
	} else {
		p = NEW(struct caserec);
		p->label   = uniq++;
		p->caseval = caseval;
		p->next    = ctx->cases;
		ctx->cases = p;
printf("L%d:\n", p->label);
		while (p = p->next)
			if (p->caseval == caseval)
				error("Duplicate case in switch");
	}
}

add_case(caseval)
{
	add_switch_case(caseval, 0);
}

add_default()
{
	add_switch_case(0, 1);
}

start_switch()
{
	int id;

	id = start_loop();

	printf("JMP L%d\n", id + 1);

	blockctx->is_switch = 1;
}

end_switch()
{
	struct caserec *p, *pnext;
	int over;
	char *cp;

	if (!blockctx)
		error("No switch to close. Cannot happen");
	if (!blockctx->is_switch)
		error("Imbalanced switch. Cannot happen");

	/*
	 *  After switchblock, jump over case selection
	 */
	printf("JMP L%d\n", over = uniq++);

	/*
	 *  Jump here from switchtop to select case to branch to.
	 */
	printf("L%d:\n", blockctx->uniq + 1);

	cp = "CASEREC0(%d,L%d)\n";
	for (p = blockctx->cases; p; p = pnext) {
		pnext = p->next;
		printf(cp, p->caseval, p->label);
		free(p);
		cp = "CASERECn(%d,L%d)\n";
	}
	if (blockctx->has_default)
		printf("JMP L%d\n", blockctx->deflabel);

	/*
	 *  After switch block and case selection.
	 */
	printf("L%d:\n", over);

	end_loop();
}

start_loop()
{
	struct blockctx *p = NEW(struct blockctx);

	p->outer = blockctx;
	p->uniq  = uniq;
	printf("L%d:\n", p->uniq);

	uniq += 2;
	blockctx  = p;

	return (p->uniq);
}

break_loop()
{
	if (!blockctx)
		error("No loop to break from");

	printf("JMP L%d\n", blockctx->uniq + 1);
}

end_loop()
{
	struct blockctx *ctx;

	if (!blockctx)
		error("No loop to close. Cannot happen");

	ctx = blockctx;
	blockctx = ctx->outer;

	printf("L%d:\n", ctx->uniq + 1);
	free(ctx);
}

