

atRi		: '@' R0 { $$ = 0; }
			| '@' R1 { $$ = 1; }
			;

Rn			: R0 { $$ = 0; }
			| R1 { $$ = 1; }
			| R2 { $$ = 2; }
			| R3 { $$ = 3; }
			| R4 { $$ = 4; }
			| R5 { $$ = 5; }
			| R6 { $$ = 6; }
			| R7 { $$ = 7; }
			;

addr 		: expr {
				$$ = $1;
			}
			;

data16		: '$' addr {
				$$ = $2;
			}
			;

data		: '$' expr {
				$$ = $2;
				/*
				 * Ugly hack to handle ~0xAA
				 */
				if (($$ & ~0xFF) == ~0xFF)
					$$ &= 0xFF;
				if (pass == 1 && ($$ < 0 || $$ > 0xFF))
					yyerror("immediate data out of range");
			}
			;

direct		: expr {
				$$ = $1;
				if (pass == 1 && ($$ & ~0xFF))
					yyerror("direct byte out of range");
			}
			| ACC    { $$ = 0xE0; }
			| B      { $$ = 0xF0; }
			| DPH    { $$ = 0x83; }
			| DPL    { $$ = 0x82; }
			| IP     { $$ = 0xB8; }
			| P0     { $$ = 0x80; }
			| P1     { $$ = 0x90; }
			| P2     { $$ = 0xA0; }
			| P3     { $$ = 0xB0; }
			| PCON   { $$ = 0x87; }
			| PSW    { $$ = 0xD0; }
			| RCAP2H { $$ = 0xCB; }
			| RCAP2L { $$ = 0xCA; }
			| SBUF   { $$ = 0x99; }
			| SCON   { $$ = 0x98; }
			| SP     { $$ = 0x81; }
			| T2CON  { $$ = 0xC8; }
			| TCON   { $$ = 0x88; }
			| TH0    { $$ = 0x8C; }
			| TH1    { $$ = 0x8D; }
			| TH2    { $$ = 0xCD; }
			| TL0    { $$ = 0x8A; }
			| TL1    { $$ = 0x8B; }
			| TL2    { $$ = 0xCC; }
			| TMOD   { $$ = 0x89; }
			;

bit			: direct '.' expr {
				$$ = ($1 < 0x80) ? (($1 - 0x20) * 8 + $3)
				                 : (($1 & 0xF8) + $3);
				if (pass == 1) {
					if ($3 < 0 || $3 > 7)
						yyerror("bit offset out of range");
					if (!(($1 >= 0x20 && $1 <= 0x2F)
					   || ($1 >= 0x80 && $1 <= 0xFF && !($1 & 7))))
						yyerror("location not bit-addressable");
				}
			}
			| expr {
				$$ = $1;
				if (pass == 1) {
					if ($$ < 0 || $$ > 7)
						yyerror("bit out of range");
				}
			}
			| AC    { $$ = 0xD6; }
			| CY    { $$ = 0xD7; }
			| EA    { $$ = 0xAF; }
			| ES    { $$ = 0xAC; }
			| ET0   { $$ = 0xA9; }
			| ET1   { $$ = 0xAB; }
			| ET2   { $$ = 0xAD; }
			| EX0   { $$ = 0xA8; }
			| EX1   { $$ = 0xAA; }
			| EXEN2 { $$ = 0xCB; }
			| EXF2  { $$ = 0xCE; }
			| F0    { $$ = 0xD5; }
			| IE0   { $$ = 0x89; }
			| IE1   { $$ = 0x8B; }
			| IT0   { $$ = 0x88; }
			| IT1   { $$ = 0x8A; }
			| OV    { $$ = 0xD2; }
			| P     { $$ = 0xD0; }
			| PS    { $$ = 0xBC; }
			| PT0   { $$ = 0xB9; }
			| PT1   { $$ = 0xBB; }
			| PT2   { $$ = 0xBD; }
			| PX0   { $$ = 0xB8; }
			| PX1   { $$ = 0xBA; }
			| RB8   { $$ = 0x9A; }
			| RCLK  { $$ = 0xCD; }
			| REN   { $$ = 0x9C; }
			| RI    { $$ = 0x98; }
			| RL2   { $$ = 0xC8; }
			| RS0   { $$ = 0xD3; }
			| RS1   { $$ = 0xD4; }
			| SM0   { $$ = 0x9F; }
			| SM1   { $$ = 0x9E; }
			| SM2   { $$ = 0x9D; }
			| TB8   { $$ = 0x9B; }
			| TCLK  { $$ = 0xCC; }
			| TF0   { $$ = 0x8D; }
			| TF1   { $$ = 0x8F; }
			| TF2   { $$ = 0xCF; }
			| TI    { $$ = 0x99; }
			| TR0   { $$ = 0x8C; }
			| TR1   { $$ = 0x8E; }
			| TR2   { $$ = 0xCA; }
			;

instruction	: ACALL addr                  { op2(0x11 | a11($2), $2); ck2k($2); }
			| ADD A ',' Rn                { op (0x28 | $4); }
			| ADD A ',' direct            { op2(0x25 , $4); }
			| ADD A ',' atRi              { op (0x26 | $4); }
			| ADD A ',' data              { op2(0x24 , $4); }
			| ADDC A ',' Rn               { op (0x38 | $4); }
			| ADDC A ',' direct           { op2(0x35 , $4); }
			| ADDC A ',' atRi             { op (0x36 | $4); }
			| ADDC A ',' data             { op2(0x34 , $4); }
			| AJMP addr                   { op2(0x01 | a11($2), $2); ck2k($2); }
			| ANL A ',' Rn                { op (0x58 | $4); }
			| ANL A ',' direct            { op2(0x55 , $4); }
			| ANL A ',' atRi              { op (0x56 | $4); }
			| ANL A ',' data              { op2(0x54 , $4); }
			| ANL direct ',' A            { op2(0x52 , $2); }
			| ANL direct ',' data         { op3(0x53 , $2, $4); }
			| ANL C ',' bit               { op2(0x82 , $4); }
			| ANL C ',' '/' bit           { op2(0xB0 , $5); }
			| CJNE A ',' direct ',' addr  { op3(0xB5 , $4, ckrel(3, $6)); }
			| CJNE A ',' data ',' addr    { op3(0xB4 , $4, ckrel(3, $6)); }
			| CJNE Rn ',' data ',' addr   { op3(0xB8 | $2, $4, ckrel(3, $6)); }
			| CJNE atRi ',' data ',' addr { op3(0xB6 | $2, $4, ckrel(3, $6)); }
			| CLR A                       { op (0xE4); }
			| CLR C                       { op (0xC3); }
			| CLR bit                     { op2(0xC2 , $2); }
			| CPL A                       { op (0xF4); }
			| CPL C                       { op (0xB3); }
			| CPL bit                     { op2(0xB2 , $2); }
			| DA A                        { op (0xD4); }
			| DEC A                       { op (0x14); }
			| DEC Rn                      { op (0x18 | $2); }
			| DEC direct                  { op2(0x15 , $2); }
			| DEC atRi                    { op (0x16); }
			| DIV AB                      { op (0x84); }
			| DJNZ Rn ',' addr            { op2(0xD8 | $2, ckrel(2, $4)); }
			| DJNZ direct ',' addr        { op3(0xD5 , $2, ckrel(2, $4)); }
			| INC A                       { op (0x04); }
			| INC Rn                      { op (0x08 | $2); }
			| INC direct                  { op2(0x05 , $2); }
			| INC atRi                    { op (0x06); }
			| INC DPTR                    { op (0xA3); }
			| JB bit ',' addr             { op3(0x20 , $2, ckrel(3, $4)); }
			| JBC bit ',' addr            { op3(0x10 , $2, ckrel(3, $4)); }
			| JC addr                     { op2(0x40 , ckrel(2, $2)); }
			| JMP '@' A '+' DPTR          { op (0x73); }
			| JNB bit ',' addr            { op3(0x30 , $2, ckrel(3, $4)); }
			| JNC addr                    { op2(0x50 , ckrel(2, $2)); }
			| JNZ addr                    { op2(0x70 , ckrel(2, $2)); }
			| JZ addr                     { op2(0x60 , ckrel(2, $2)); }
			| LCALL addr                  { op3(0x12 , $2 >> 8, $2); }
			| LJMP addr                   { op3(0x02 , $2 >> 8, $2); }
			| MOV A ',' Rn                { op (0xE8 | $4); }
			| MOV A ',' direct            { op2(0xE5 , $4); }
			| MOV A ',' atRi              { op (0xE6 | $4); }
			| MOV A ',' data              { op2(0x74 , $4); }
			| MOV Rn ',' A                { op (0xF8 | $2); }
			| MOV Rn ',' direct           { op2(0xA8 | $2, $4); }
			| MOV Rn ',' data             { op2(0x78 | $2, $4); }
			| MOV direct ',' A            { op2(0xF5 , $2); }
			| MOV direct ',' Rn           { op2(0x88 | $4, $2); }
			| MOV direct ',' direct       { op3(0x85 , $4, $2); }
			| MOV direct ',' atRi         { op2(0x86 | $4, $2); }
			| MOV direct ',' data         { op3(0x75 , $2, $4); }
			| MOV atRi ',' A              { op (0xF6 | $2); }
			| MOV atRi ',' direct         { op2(0xA6 | $2, $4); }
			| MOV atRi ',' data           { op2(0x76 | $2, $4); }
			| MOV C ',' bit               { op2(0xA2 , $4); }
			| MOV bit ',' C               { op2(0x92 , $2); }
			| MOV DPTR ',' data16         { op3(0x90 , $4 >> 8, $4); }
			| MOVC A ',' '@' A '+' DPTR   { op (0x93); }
			| MOVC A ',' '@' A '+' PC     { op (0x83); }
			| MOVX A ',' atRi             { op (0xE2 | $4); }
			| MOVX A ',' '@' DPTR         { op (0xE0); }
			| MOVX atRi ',' A             { op (0xF2 | $4); }
			| MOVX '@' DPTR ',' A         { op (0xF0); }
			| MUL AB                      { op (0xA4); }
			| NOP                         { op (0x00); }
			| ORL A ',' Rn                { op (0x48 | $4); }
			| ORL A ',' direct            { op2(0x45 , $4); }
			| ORL A ',' atRi              { op (0x46 | $4); }
			| ORL A ',' data              { op2(0x44 , $4); }
			| ORL direct ',' A            { op2(0x42 , $2); }
			| ORL direct ',' data         { op3(0x43 , $2, $4); }
			| ORL C ',' bit               { op2(0x72 , $4); }
			| ORL C ',' '/' bit           { op2(0xA0 , $5); }
			| POP direct                  { op2(0xD0 , $2); }
			| PUSH direct                 { op2(0xC0 , $2); }
			| RET                         { op (0x22); }
			| RETI                        { op (0x32); }
			| RL A                        { op (0x23); }
			| RLC A                       { op (0x33); }
			| RR A                        { op (0x03); }
			| RRC A                       { op (0x13); }
			| SET C                       { op (0xD3); }
			| SET bit                     { op2(0xD2 , $2); }
			| SJMP addr                   { op2(0x80 , ckrel(2, $2)); }
			| SUBB A ',' Rn               { op (0x98 | $4); }
			| SUBB A ',' direct           { op2(0x95 , $4); }
			| SUBB A ',' atRi             { op (0x96 | $4); }
			| SUBB A ',' data             { op2(0x94 , $4); }
			| SWAP A                      { op (0xC4); }
			| XCH A ',' Rn                { op (0xC8 | $4); }
			| XCH A ',' direct            { op2(0xC5 , $4); }
			| XCH A ',' atRi              { op (0xC6 | $4); }
			| XCHD A ',' atRi             { op (0xD6 | $4); }
			| XRL A ',' Rn                { op (0x68 | $4); }
			| XRL A ',' direct            { op2(0x65 , $4); }
			| XRL A ',' atRi              { op (0x66 | $4); }
			| XRL A ',' data              { op2(0x64 , $4); }
			| XRL direct ',' A            { op2(0x62 , $2); }
			| XRL direct ',' data         { op3(0x63 , $2, $4); }
			;

%%

a11(a)
{
	return (((a >> 8) << 5) & 0xE0);
}

ck2k(a)
{
	if (pass == 1 && ((a & 0xF800) != (dot & 0xF800)))
		yyerror("absolute out of range");
}

ckrel(oplen, a)
{
	int rel = a - (dot + oplen);

	if (pass == 1 && (rel < -127 || rel > 127))
		yyerror("relative out of range");
	return (rel);
}

static char *vers2 = "@(#)i8051.y 970803";

char *cpu_family = "i8051";
int MSB_first = 0;

